@@ -5,11 +5,51 @@ CGI::Application::Plugin::REST - Helps implement RESTful architecture in CGI app
 
 =head1 SYNOPSIS
 
-    use CGI::Application::Plugin::REST qw( :all );
+    package WidgetView;
+    use base qw( CGI::Application );
+    use CGI::Application::Plugin::REST qw( rest_route rest_param );
+
+    sub setup {
+        my ($self) = @_;
+
+        $self->rest_route({
+            '/widget'                   => {
+                'GET'    => 'showlist',
+                'POST'   => {
+                    'application/xml' => 'new_widget',
+                },
+            },
+            '/widget/:id'               => {
+                'GET'    => 'showdetail',
+            },
+        };
+    }
+
+    sub new_widget {
+        my ($self) = @_;
+
+        # make a new widget
+   }
+
+    sub showdetail {
+        my ($self) = @_;
+
+        my $id = $self->rest_param('id');
+
+        # display the widget with the id $id.
+   }
+
+    sub showlist {
+        my ($self) = @_;
+
+        # show the entire list of widgets.
+   }
+
+   1;
 
 =head1 ABSTRACT
 
-If you use the L<CGI::Application> framework, this plugin will help you create
+If you use the L<CGI::Application|CGI::Application> framework, this plugin will help you create
 a RESTful (that's the common term for "using REST") architecture by
 abstracting out a lot of the busy work needed to make it happen.
 
@@ -25,14 +65,15 @@ use REST::Utils qw/ media_type request_method /;
 
 =head1 VERSION
 
-This document describes CGI::Application::Plugin::REST Version 0.2
+This document describes CGI::Application::Plugin::REST Version 0.3
 
 =cut
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 
 our @EXPORT_OK =
-  qw/ rest_error_mode rest_param rest_resource rest_route rest_route_prefix /;
+  qw/ rest_error_mode rest_param rest_resource rest_route rest_route_info
+  rest_route_prefix /;
 
 our %EXPORT_TAGS = ( 'all' => [@EXPORT_OK] );
 
@@ -44,21 +85,28 @@ Wide Web such as URIs, MIME media types, and HTTP instead of building up
 protocols and functions on top of them.
 
 This plugin contains a number of functions to support the various REST
-concepts. They try to use existing L<CGI::Application> functionality
+concepts. They try to use existing L<CGI::Application|CGI::Application> functionality
 wherever possible.
 
-C<use>'ing this plugin will intercept L<CGI::Application>'s standard dispatch
+C<use>'ing this plugin will intercept L<CGI::Application|CGI::Application>'s standard dispatch
 mechanism.  Instead of being selected based on a query parameter like C<rm>,
 the run mode will be determined by comparing URI patterns defined in your app
-with the L<rest_route()> method.  (Referred from here on, as "routes".)
+with the L<rest_route|rest_route()> method.  (Referred from here on, as "routes".)
 Optionally, specific HTTP methods or MIME media types can be defined in a
 route too.  One by one, each entry in the reverse asciibetically sorted table
 of defined routes is compared to the incoming HTTP request and the first
 successful match is selected.  The run mode mapped to that route is then
 called.
 
-This is done via overriding L<CGI::Application>'s C<mode_param()> function so
-it should be compatible with other L<CGI::Application> plugins.
+This is done via overriding L<CGI::Application|CGI::Application>'s C<mode_param()> function so
+it should be compatible with other L<CGI::Application|CGI::Application> plugins.
+
+=head2 DevPopup Support
+
+If you are using L<C::A::P::DevPopup|CGI::Application::Plugin::DevPopup> (i.e. the environment
+variable C<CAP_DEVPOPUP_EXEC> is set,) C<use>'ing this module will register a
+callback which will add debug information about the current route (See L<rest_route_info|rest_route_info()>),
+parameters (See L<rest_param|rest_param()>) etc.
 
 =head1 FUNCTIONS
 
@@ -140,8 +188,8 @@ sub _rest_dispatch {
 
     # get the module name from the table
     if ( !exists $self->{'__rest_dispatch_table'} ) {
-        $self->header_add( -status => '400 No Dispatch Table' );
-        return rest_error_mode($self);
+        $self->header_add( -status => '500 No Dispatch Table' );
+        return rest_error_mode( $self, $EVAL_ERROR );
     }
 
     # look at each rule and stop when we get a match
@@ -150,7 +198,7 @@ sub _rest_dispatch {
         my @names = ();
 
         # $rule will be transformed later so save the original form first.
-        my $key = $rule;
+        my $origrule = $rule;
         $rule = rest_route_prefix($self) . $rule;
 
         # translate the rule into a regular expression, but remember where
@@ -161,15 +209,15 @@ sub _rest_dispatch {
         # and then remember which position it matches
         $rule =~ s{
                         (^ | /)                 # beginning or a /
-                        (: ([^/\?]+) (\?)?)     # stuff in between
+                        (: ([^/?]+) ([?])?)     # stuff in between
                 }{
                         push(@names, $3);
                         $1 . ( $4 ? '?( [^/]* )?' : '([^/]*)')
                 }egsmx;
 
         # '/*' onwards will become '(.*)\$'
-        if ( $rule =~ m{/\* .* $}msx ) {
-            $rule =~ s{(/\* .* )$}{/(.*)\$}msx;
+        if ( $rule =~ m{/[*] .* $}msx ) {
+            $rule =~ s{(/[*] .* )$}{/(.*)\$}msx;
             push @names, 'dispatch_uri_remainder';
         }
 
@@ -177,7 +225,7 @@ sub _rest_dispatch {
         if ( my @values = ( $path =~ m{^$rule$}msx ) ) {
 
             $self->{'__match'} = $path;
-            my $table = $self->{'__rest_dispatch_table'}->{$key};
+            my $table = $self->{'__rest_dispatch_table'}->{$origrule};
 
             # next check request method.
             my $method = request_method($q);
@@ -193,21 +241,22 @@ sub _rest_dispatch {
                     -status => "405 Method '$method' Not Allowed",
                     -allow  => ( join q{, }, sort keys %{$table} ),
                 );
-                return rest_error_mode($self);
+                return rest_error_mode( $self, $EVAL_ERROR );
             }
 
             # then check MIME media type
             my @types = keys %{$table};
             my $preferred = media_type( $q, \@types );
-            if ( !defined $preferred ) {
-                $self->header_add( -status => '415 Unsupported Media Type' );
-                return rest_error_mode($self);
-            }
-            if ( $preferred eq q{} ) {
+            if ( !defined $preferred || $preferred eq q{} ) {
                 $preferred = q{*/*};
             }
             my $rm_name = $table->{$preferred};
 
+            if ( !defined $rm_name ) {
+                $self->header_add( -status => '415 Unsupported Media Type' );
+                return rest_error_mode( $self, $EVAL_ERROR );
+            }
+
             my $sub;
             if ( ref $rm_name eq 'CODE' ) {
                 $sub = $self->$rm_name;
@@ -216,9 +265,9 @@ sub _rest_dispatch {
                 $sub = eval { return $self->can($rm_name); };
             }
             if ( !defined $sub ) {
-                $self->header_add( -status =>
-                      "501 Method '$method' Not Implemented by $rm_name" );
-                return rest_error_mode($self);
+                $self->header_add(
+                    -status => "501 Function '$rm_name' Doesn't Exist" );
+                return rest_error_mode( $self, $EVAL_ERROR );
             }
 
             $self->param( 'rm', $rm_name );
@@ -232,7 +281,7 @@ sub _rest_dispatch {
 
             $self->{'__r_params'} = {
                 'path_received' => $path,
-                'rule_matched'  => $rule,
+                'rule_matched'  => $origrule,
                 'runmode'       => $rm_name,
                 'method'        => $method,
                 'mimetype'      => $preferred,
@@ -243,10 +292,10 @@ sub _rest_dispatch {
     }
 
     $self->header_add( -status => '404 No Route Found' );
-    return rest_error_mode($self);
+    return rest_error_mode( $self, $EVAL_ERROR );
 }
 
-=head3 rest_error_mode()
+=head2 rest_error_mode()
 
 This function gets or sets the run mode which is called if an error occurs
 during the dispatch process.  In this run mode, you can do whatever error
@@ -259,6 +308,17 @@ Example 1:
     $self->rest_error_mode('my_error_mode');
     my $em = $self->rest_error_mode; # $em equals 'my_error_mode'.
 
+Why isn't the standard L<CGI::Application|CGI::Application> error mode mechanism used? The
+problem is that at the point L<C::A::P::REST|CGI::Application::Plugin::REST>
+plugs into the dispatch process, the error mode has not been defined.  You
+might also want to use L<rest_error_mode|rest_error_mode()> in your own code to do a different
+sort of handling for errors in your REST API (which will typically only
+require setting the HTTP status code) as opposed to handling for end user
+errors.
+
+Your rest_error_mode handler function will receive as a parameter the value of C<$@>
+if any.
+
 =cut
 
 sub rest_error_mode {
@@ -278,10 +338,10 @@ sub rest_error_mode {
     return $self->{'__rest_error_mode'};
 }
 
-=head3 rest_param()
+=head2 rest_param()
 
 The C<rest_param> function is used to retrieve or set named parameters
-defined by the L<rest_route()> function. it can be called in three ways.
+defined by the L<rest_route|rest_route()> function. it can be called in three ways.
 
 =over 4
 
@@ -314,7 +374,7 @@ You could also use a hashref.
 The value of a parameter need not be a scalar, it could be any any sort of
 reference even a coderef.
 
-    $self->rest_param(number => &pick_a_random_number);
+    $self->rest_param(number => \&pick_a_random_number);
 
 In this case, the function does not return anything.
 
@@ -356,7 +416,7 @@ sub rest_param {
     return;
 }
 
-=head3 rest_resource()
+=head2 rest_resource()
 
 This function will set up a complete REST API for a collection of items with all
 the CRUD (Create, Read, Update, Delete) operations in one call.  A collection
@@ -367,12 +427,13 @@ Example 1: basic usage of rest_resource()
 
     $self->rest_resource('widget');
 
-is exactly equal to the following invocation of L<rest_route()>:
+is exactly equal to the following invocation of L<rest_route|rest_route()>:
 
     $self->rest_route(
         '/widget'                   => {
-            'GET'    => 'widget_index',
-            'POST'   => 'widget_create',
+            'GET'     => 'widget_index',
+            'POST'    => 'widget_create',
+            'OPTIONS' => 'widget_options',
         },
         '/widget/:id'               => {
             'DELETE' => 'widget_destroy',
@@ -423,6 +484,11 @@ Should be used to display resource with the id C<:id>.
 
 Should be used to alter the existing resource with the id C<:id>.
 
+=item *_options
+
+Should be used to retrieve metadata that describes the resource's available
+interactions.
+
 =back
 
 Various aspects of the generated routes can be customized by passing this
@@ -435,13 +501,13 @@ method a hash (or hashref) of parameters instead of a scalar.
 This parameter is required.  It is used to form the URI the route will match
 to.
 
-HINT: use L<rest_route_prefix> for more complicated URIs.
+HINT: use L<rest_route_prefix|rest_route_prefix()> for more complicated URIs.
 
 =item identifier
 
 This parameter sets the name assigned to the unique identifier of an item in
 the collection which is used in some generated routes.  It can be retrieved
-with L<rest_param>.  It defaults to C<id>.
+with L<rest_param|rest_param()>.  It defaults to C<id>.
 
 =item prefix
 
@@ -452,9 +518,9 @@ to C<resource>.
 
 Both these parameters represent arrayrefs of MIME media types.  C<in_type>
 defines acceptable MIME media types for data incoming to your API (i.e.
-C<POST>S and C<PUT>s) and C<out_type> does the same for outgoing data (i.e.
-C<GET>s.) C<DELETE> requests do not need MIME media types so they are not
-covered.
+C<POST>s and C<PUT>s) and C<out_type> does the same for outgoing data (i.e.
+C<GET>s and C<OPTIONS>.) C<DELETE> requests do not need MIME media types so
+they are not covered.
 
 The reason there are two separate parameters is that typically the number of
 data formats a REST API will serve is different to the number and kind of
@@ -470,7 +536,7 @@ Example 2: advanced usage of rest_resource()
         identifier => 'num', in_types => [ 'application/xml' ],
         out_types => [ 'text/html', 'text/plain' ], );
 
-is equal to the following invocation of L<rest_route()>:
+is equal to the following invocation of L<rest_route|rest_route()>:
 
     $self->rest_route(
         '/fidget'                   => {
@@ -508,9 +574,9 @@ is equal to the following invocation of L<rest_route()>:
         },
     );
 
-If you need more complicated mappings then this, use L<rest_route()>.
+If you need more complicated mappings then this, use L<rest_route|rest_route()>.
 
-L<rest_resource()> returns the map of routes and handlers that was created.
+L<rest_resource|rest_resource()> returns the map of routes and handlers that was created.
 
 =cut
 
@@ -555,6 +621,8 @@ sub rest_resource {
         "/$resource" => {
             'GET'  => _make_resource_route( $prefix . '_index',  $out_types ),
             'POST' => _make_resource_route( $prefix . '_create', $in_types ),
+            'OPTIONS' =>
+              _make_resource_route( $prefix . '_options', $out_types ),
         },
         "/$resource/:$id" => {
             'DELETE' => _make_resource_route( $prefix . '_destroy', [q{*/*}] ),
@@ -614,10 +682,10 @@ sub _make_resource_route {
     return $ret;
 }
 
-=head3 rest_route()
+=head2 rest_route()
 
 When this function is given a hash or hashref, it configures the mapping of
-routes to handlers (run modes within your L<CGI::Application>).
+routes to handlers (run modes within your L<CGI::Application|CGI::Application>).
 
 It returns the map of routes and handlers.
 
@@ -642,9 +710,23 @@ A segment in a route is matched literally.  So if a request URI matches
 http://localhost/foo, the run mode that handles the route in example 1 will
 be used.
 
+If you want to match the URI base itself, you can do it like this:
+
+Example 2: route to a URI base
+
+    /
+
+This matches C<http://localhost/>.  Some people don't like the trailing slash;
+they can be accomodated by using an empty string as the route as in Example 3.
+
+Example 3: route to a URI base without the trailing /
+    ''
+
+This matches C<http://localhost>.
+
 Routes can have more complex specifications.
 
-Example 2: a more complex route
+Example 4: a more complex route
 
     /bar/:name/:id?/:email
 
@@ -652,7 +734,7 @@ If a segment of a route is prefixed with a :, it is not matched literally but
 treated as a parameter name.  The value of the parameter is whatever actually
 got matched.  If the segment ends with a ?, it is optional otherwise it is
 required.  The values of these named parameters can be retrieved with the
-L<rest_param()> method.
+L<rest_param|rest_param()> method.
 
 In example 2, http://localhost/bar/jaldhar/76/jaldhar@braincells.com would
 match.  C<rest_param('name')> would return 'jaldhar',  C<rest_param('id')>
@@ -671,13 +753,13 @@ parameter ':email' is missing.
 Note: Each named parameter is returned as a scalar.  If you want ':email' to
 actually be an email address, it is up to your code to validate it before use.
 
-Example 3: a wild card route
+Example 5: a wild card route
 
     /baz/string/*
 
 If the route specification contains /*, everything from then on will be
 put into the special parameter 'dispatch_uri_remainder' which you can retrieve
-with L<rest_param()> just like any other parameter.  Only one wildcard can
+with L<rest_param|rest_param()> just like any other parameter.  Only one wildcard can
 be specified per route.  Given the request URI
 http://localhost/baz/string/good, C<rest_param('dispatch_uri_remainder')>
 would return 'good', with http://localhost/baz/string/evil it would return
@@ -725,24 +807,26 @@ Example 5: More complex handlers
     );
 
 If the handler is a hashref, the keys of the second-level hash are HTTP
-methods and the values if scalars or coderefs, are run modes.  The key
-can also be * which matches all methods not explicitly specified.  If a valid
-method cannot be matched, an error is raised and the HTTP status of the
-response is set to 405.  (See L<"DIAGNOSTICS">)
+methods and the values if scalars or coderefs, are run modes.  Supported
+methods are C<HEAD>, C<GET>, C<POST>, C<PUT>, C<DELETE>, and C<OPTIONS>.  The key can also be C<*>
+which matches all methods not explicitly specified.  If a valid method cannot
+be matched, an error is raised and the HTTP status of the response is set to
+405.  (See L<"DIAGNOSTICS">.)
 
 In example 5, a C<GET> request to http://localhost/quux will be dispatched to
 C<ptang()>.  A C<DELETE> to http://localhost/quux will dispatch to C<krrang()>.
-A C<POST>, C<PUT> or C<HEAD> will cause an error.
+An C<OPTIONS>) C<POST>, C<PUT> or C<HEAD> will cause an error.
 
 A C<POST> request to http://localhost/edna will dispatch to C<zip()>
 while any other type of request to that URL will dispatch to C<blop()>
 
 The values of the second-level hash can also be hashes.  In this case the keys
 of the third-level hash represent MIME media types.  The values are run modes.
-The best possible match is made use C<best_match()> from L<REST::Utils>.
+The best possible match is made use C<best_match()> from L<REST::Utils|REST::Utils>.
 according to the HTTP Accept header sent in the request.  If a valid MIME
-media type cannot be matched an error is raised and the HTTP status of the
-response is set to 415.  (See L<"DIAGNOSTICS">)
+media type cannot be matched C<*/*> is tried as a last resort.  If there is no
+handler for even that, an error is raised and the HTTP status of the response
+is set to 415.  (See L<"DIAGNOSTICS">)
 
 In example 5, a C<GET> request to http://localhost/grudnuk with MIME
 media type application/xml will dispatch to C<zip()>. If the same request is
@@ -759,10 +843,6 @@ response is set to 404 (See L<"DIAGNOSTICS">.)
 sub rest_route {
     my ( $self, @routes ) = @_;
 
-    if ( !exists $self->{'__rest_dispatch_table'} ) {
-        $self->{'__rest_dispatch_table'} = { q{/} => 'dump_html' };
-    }
-
     my $rr_m = $self->{'__rest_dispatch_table'};
 
     my $num_routes = scalar @routes;
@@ -808,7 +888,8 @@ sub _method_hashref {
               ') has an invalid route definition';
         }
 
-        my @request_methods = ( 'GET', 'POST', 'PUT', 'DELETE', 'HEAD', q{*} );
+        my @request_methods =
+          ( 'GET', 'POST', 'PUT', 'DELETE', 'HEAD', 'OPTIONS', q{*}, );
         foreach my $req (@methods) {
             if ( scalar grep { $_ eq $req } @request_methods ) {
                 my $subroute = $routes->{$rule}->{$req};
@@ -851,7 +932,62 @@ sub _mime_hashref {
     return;
 }
 
-=head3 rest_route_prefix()
+=head2 rest_route_info()
+
+This function can be called in a route handler.  It returns a reference to a
+hash which contains some information about the current route.
+
+=over 4
+
+=item path_received
+
+The value of the C<PATH_INFO> environment variable.
+
+=item rule_matched
+
+The rule that was successfully matched to determine this route.
+
+=item runmode
+
+The name of the function being called by this route.
+
+=item method
+
+The HTTP method that was matched by this route.
+
+=item mimetype
+
+The MIME media type that was matched by this route.
+
+=back
+
+Example 1:
+
+    $self->rest_route(
+         '/foo' => {
+             'GET' => 'bar',
+         },
+    );
+
+    ...
+
+    sub bar() {
+        my ($self) = @_;
+
+        my $info = $self->rest_route_info;
+
+        say $info->{method};  # prints 'GET'
+	}
+
+=cut
+
+sub rest_route_info {
+    my ($self) = @_;
+
+    return $self->{'__r_params'};
+}
+
+=head2 rest_route_prefix()
 
 Use this function to set a prefix for routes to avoid unnecessary repetition
 when you have a number of similar ones.
@@ -892,41 +1028,90 @@ sub rest_route_prefix {
 
 }
 
+=head1 OTHER DISPATCH PLUGINS COMPARED
+
+There are several other modules that allow L<CGI::Application|CGI::Application> to dispatch to
+a run mode based on the C<PATH_INFO> environment variable instead of the
+traditional CGI parameter.  They each take a markedly different approach to
+implementation.  Here is a comparison.
+
+Executive summary:  L<C::A::P::REST|CGI::Application::Plugin::REST> is the best :-)
+
+=head2 L<CGI::Application|CGI::Application> Itself
+
+You can set the run mode with the C<path_info> option to C<mode_param()>.
+This is limited to one segment (i.e. between C</>'s) of the path info.
+
+Dispatch based on HTTP method or MIME media type is not supported.
+
+=head2 L<CGI::Application::Dispatch|CGI::Application::Dispatch>
+
+This module has influenced most of the other dispatchers including this one.
+It replaces L<CGI::Application|CGI::Application> as the base class for your application.
+
+It has extensive capabilities for matching path info.  It can capture variable
+segments in the URI with : ? and * tokens. They are retrievable in run
+modes as L<CGI::Application|CGI::Application> parameters (i.e. via C<$self-E<gt>param()>.
+
+You can also dispatch by HTTP method but not by MIME media type.  The HTTP
+method is determined by looking at the C<HTTP_REQUEST_METHOD> environment
+variable only.  Methods called C<auto_rest()> and C<auto_rest_lc()> append the
+the HTTP method (all upper case and all lower case respectively) to a run mode
+that is determined by a dispatch rule which provides a limited version of
+L<C::A::P::REST|CGI::Application::Plugin::REST>'s L<rest_resource|rest_resource()> function.
+
+=head2 L<C::A::P::ActionDispatch|CGI::Application::Plugin::ActionDispatch>
+
+This module adds an attribute handler to run modes of your choice which enable
+parsing of the path info with regular expressions and dispatch to the run mode
+matched. Capturing parentheses in the regex can be accessed via the
+C<action_args()> method.
+
+Dispatch based on HTTP method or MIME media type is not supported.
+
+=head2 L<C::A::P::Routes|CGI::Application::Plugin::Routes>
+
+This module installs a prerun hook that matches path info segments with support
+for capturing variable with the : ? and * tokens.  They are retrievable in run
+modes as L<CGI|CGI> parameters (i.e. via C<$self-E<gt>query-E<gt>param()>
+
+Dispatch based on HTTP method or MIME media type is not supported.
+
 =head1 DIAGNOSTICS
 
 During the dispatch process, errors can occur in certain circumstances. If an
 error occurs the appropriate HTTP status is set and execution passes to the
-run mode set by L<"rest_error_mode()">.  Here is a list of status codes and
+run mode set by L<rest_error_mode|rest_error_mode()>.  Here is a list of status codes and
 messages.
 
 =over 4
 
-=item * 400 No Dispatch Table
-
-This error can occur if L<rest_route()> was not called.
-
 =item * 404 No Route Found
 
 None of the specified routes matched the request URI.
 
 =item * 405 Method '$method' Not Allowed
 
-The route you specified with L<rest_route()> does not allow this HTTP
+The route you specified with L<rest_route|rest_route()> does not allow this HTTP
 request method.  An HTTP C<Allow> header is added to the response specifying
 which methods can be used.
 
 =item * 415 Unsupported Media Type
 
 None of the MIME media types requested by the client can be returned by this
-route.
+route and there is no handler for C<*/*>.
+
+=item * 500 No Dispatch Table
+
+This error can occur if L<rest_route|rest_route()> was not called.
 
 =item * 500 Application Error
 
 The function that was called for this run_mode C<die>'d somewhere.
 
-=item * 501 Function Doesn't Exist
+=item * 501 Function '$function_name' Doesn't Exist
 
-The function that you wanted to call from L<rest_route()> for this run_mode
+The function that you wanted to call from L<rest_route|rest_route()> for this run_mode
 doesn't exist in your application.
 
 =back
@@ -945,16 +1130,16 @@ your bug as I make changes.
 
 =over 4
 
-=item * L<CGI::Application>:
+=item * L<CGI::Application|CGI::Application>:
 
 The application framework this module plugs into.
 
-=item * L<REST::Utils>:
+=item * L<REST::Utils|REST::Utils>:
 
-L<CGI::Application::Plugin::REST> uses my L<REST::Utils> module behind the
+L<C::A::P::REST|CGI::Application::Plugin::REST> uses my L<REST::Utils|REST::Utils> module behind the
 scenes.
 
-=item * L<REST::Application>:
+=item * L<REST::Application|REST::Application>:
 
 This module by Matthew O'Connor gave me some good ideas.
 
@@ -971,8 +1156,8 @@ understanding the ins and outs of REST.
 
 =head1 THANKS
 
-Much of the code in this module is based on L<CGI::Application::Plugin:Routes>
-by JuliE<aacute>n Porta who in turn credits Michael Peter's L<CGI::Application::Dispatch>.
+Much of the code in this module is based on L<C::A::P::Routes|CGI::Application::Plugin:Routes>
+by JuliE<aacute>n Porta who in turn credits Michael Peter's L<CGI::Application::Dispatch|CGI::Application::Dispatch>.
 
 =head1 AUTHOR
 
